package com.alterdekim.hearthhack.component.processor.client.request.generic;

import com.alterdekim.Protocol;
import com.alterdekim.hearthhack.component.TcpConnection;
import com.alterdekim.hearthhack.util.GetAccountInfoRequest;
import lombok.extern.slf4j.Slf4j;
import org.reflections.Reflections;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

@Slf4j
public class GeneralGenericParser {

    private Map<GetAccountInfoRequest, GenericParser> parsers;

    public GeneralGenericParser() {
        init();
    }

    private void init() {
        this.parsers = new HashMap<>();
        Set<Class<? extends GenericParser>>  classes = new Reflections(this.getClass().getPackageName()).getSubTypesOf(GenericParser.class);
        classes.forEach(c -> {
            try {
                var ci = c.getDeclaredConstructor().newInstance();
                this.parsers.put(ci.getId(), ci);
            } catch (Exception e) {
                log.error(e.getMessage());
            }
        });
    }

    public void parseGenericRequest(int token, Protocol.UtilGenericRequest request, TcpConnection conn) {
        GetAccountInfoRequest req = GetAccountInfoRequest.parseFromInt(request.getRequestSubId());
        if( !this.parsers.containsKey(req) ) {
            log.warn("Unknown generic request: {}", req);
            return;
        }
        try {
            this.parsers.get(req).parseGenericRequest(token, conn);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }
}
